import { SendServerMessage } from 'common-ts/index.js';
import { getMessageBody } from 'common-ts/util/message-util.js';
import { validateISODateString, validateNumber, validateString } from 'common-ts/util/validation-util.js';
import { validateEnumOutcomes } from './util/validation-util.js';
// Expose all 'common' endpoints
export * from 'common-ts/index.js';
/** Specific Oracle Server message functions */
export function GetPublicKey() {
    console.debug('GetPublicKey()');
    const m = getMessageBody("getpublickey" /* MessageType.getpublickey */);
    return SendServerMessage(m).then(response => {
        return response;
    });
}
export function GetStakingAddress() {
    console.debug('GetStakingAddress()');
    const m = getMessageBody("getstakingaddress" /* MessageType.getstakingaddress */);
    return SendServerMessage(m).then(response => {
        return response;
    });
}
export function ExportStakingAddressWIF() {
    console.debug('ExportStakingAddressWIF()');
    const m = getMessageBody("exportstakingaddresswif" /* MessageType.exportstakingaddresswif */);
    return SendServerMessage(m).then(response => {
        return response;
    });
}
export function ListAnnouncements() {
    console.debug('ListAnnouncements()');
    const m = getMessageBody("listannouncements" /* MessageType.listannouncements */);
    return SendServerMessage(m).then(response => {
        return response;
    });
}
export function SignMessage(message) {
    console.debug('SignMessage()', message);
    validateString(message, 'SignMessage()', 'message');
    const m = getMessageBody("signmessage" /* MessageType.signmessage */, [message]);
    return SendServerMessage(m).then(response => {
        return response;
    });
}
export function GetAnnouncement(eventName) {
    console.debug('GetAnnouncement()', eventName);
    validateString(eventName, 'GetAnnouncement()', 'eventName');
    const m = getMessageBody("getannouncement" /* MessageType.getannouncement */, [eventName]);
    return SendServerMessage(m).then(response => {
        return response;
    });
}
export function CreateEnumAnnouncement(eventName, maturationTimeISOString, outcomes) {
    console.debug('CreateEnumAnnouncement()', eventName, maturationTimeISOString, outcomes);
    validateString(eventName, 'CreateEnumAnnouncement()', 'eventName');
    validateISODateString(maturationTimeISOString, 'CreateEnumAnnouncement()', 'maturationTimeISOString');
    validateEnumOutcomes(outcomes, 'CreateEnumAnnouncement()');
    const m = getMessageBody("createenumannouncement" /* MessageType.createenumannouncement */, [eventName, maturationTimeISOString, outcomes]);
    return SendServerMessage(m).then(response => {
        return response;
    });
}
export function CreateNumericAnnouncement(eventName, maturationTimeISOString, minValue, maxValue, unit, precision) {
    console.debug('CreateNumericAnnouncement()', eventName, maturationTimeISOString, minValue, maxValue, unit, precision);
    validateString(eventName, 'CreateNumericAnnouncement()', 'eventName');
    validateISODateString(maturationTimeISOString, 'CreateNumericAnnouncement()', 'maturationTimeISOString');
    validateNumber(minValue, 'CreateNumericAnnouncement()', 'minValue');
    validateNumber(maxValue, 'CreateNumericAnnouncement()', 'maxValue');
    // TODO : Validate minValue/maxValue?
    validateString(unit, 'CreateNumericAnnouncement()', 'unit'); // unit can be an empty string
    validateNumber(precision, 'CreateNumericAnnouncement()', 'precision');
    if (precision < 0)
        throw (Error('CreateNumericAnnouncement() precision must be >= 0'));
    const m = getMessageBody("createnumericannouncement" /* MessageType.createnumericannouncement */, [eventName, maturationTimeISOString, minValue, maxValue, unit, precision]);
    return SendServerMessage(m).then(response => {
        return response;
    });
}
export function SignEnum(eventName, outcome) {
    console.debug('SignEnum()', eventName, outcome);
    validateString(eventName, 'SignEnum()', 'eventName');
    validateString(outcome, 'SignEnum()', 'outcome');
    const m = getMessageBody("signenum" /* MessageType.signenum */, [eventName, outcome]);
    return SendServerMessage(m).then(response => {
        return response;
    });
}
export function SignDigits(eventName, outcome) {
    console.debug('SignDigits()', eventName, outcome);
    validateString(eventName, 'SignDigits()', 'eventName');
    validateNumber(outcome, 'SignDigits()', 'outcome');
    const m = getMessageBody("signdigits" /* MessageType.signdigits */, [eventName, outcome]);
    return SendServerMessage(m).then(response => {
        return response;
    });
}
export function GetSignatures(eventName) {
    console.debug('GetSignatures()', eventName);
    validateString(eventName, 'GetSignatures()', 'eventName');
    const m = getMessageBody("getsignatures" /* MessageType.getsignatures */, [eventName]);
    return SendServerMessage(m).then(response => {
        return response;
    });
}
export function DeleteAnnouncement(eventName) {
    console.debug('DeleteAnnouncement()', eventName);
    validateString(eventName, 'DeleteAnnouncement()', 'eventName');
    const m = getMessageBody("deleteannouncement" /* MessageType.deleteannouncement */, [eventName]);
    return SendServerMessage(m).then(response => {
        return response;
    });
}
export function DeleteAttestation(eventName) {
    console.debug('DeleteAttestation()', eventName);
    validateString(eventName, 'DeleteAttestation()', 'eventName');
    const m = getMessageBody("deleteattestation" /* MessageType.deleteattestation */, [eventName]);
    return SendServerMessage(m).then(response => {
        return response;
    });
}
export function GetOracleName() {
    console.debug('GetOracleName()');
    const m = getMessageBody("getoraclename" /* MessageType.getoraclename */);
    return SendServerMessage(m).then(response => {
        return response;
    });
}
export function SetOracleName(oracleName) {
    console.debug('SetOracleName()', oracleName);
    const m = getMessageBody("setoraclename" /* MessageType.setoraclename */, [oracleName]);
    return SendServerMessage(m).then(response => {
        return response;
    });
}
//# sourceMappingURL=index.js.map